﻿// <summary>ソースコード：ダム（一定量、定率定量方式）モデル要素演算データクラス</summary>
// <author>CommonMP</author>

using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Controller;

// ToDo namespace は　モデル開発者が変更して、ユニークな名称にして下さい
namespace jp.or.jccajsce.DamConstantRateDischargingModel
{
    /// <summary><para>class outline:</para>
    /// <para>ダム（一定量、定率定量方式）モデル要素内演算中データクラス</para>
    /// </summary>
    /// <remarks><para>history:</para>
    /// <para>[CommonMP][ver 1.0.0][2014/08][新規作成]</para>
    /// <para>remarks</para>
    /// <para>演算中の状態、初期条件等を保持するデータクラス</para>
    /// </remarks>
    [Serializable]
    public class DamConstantRateDischargingCalInfo : McCalInfo
    {

        // 計算に使用する変数

        /// <summary>カット開始流量</summary>
        public double m_dQc = 0;
        /// <summary>放流率</summary>
        public double m_da = 0;
        /// <summary>計画最大放流量</summary>
        public double m_dQmax = 0;
        /// <summary>ただし書き操作開始水位</summary>
        public double m_dDamH1 = 0;
        /// <summary>後放流あり・なし</summary>
        public bool m_bato = false;
        /// <summary>洪水期制限水位</summary>
        public double m_dDamH2 = 0;
        /// <summary>H-V関係のファイル名</summary>
        public string m_sHvFile = "";
        /// <summary>H-Vのデータ組数</summary>
        public long m_ldamHV = 0;
        /// <summary>H-Vのデータ</summary>
        public double[] m_ddamH = null;
        /// <summary>H-Vのデータ</summary>
        public double[] m_ddamV = null;
        /// <summary>初期貯水位</summary>
        public double m_dInit = 0;

        /// <summary>放流量</summary>
        public double m_dQout = 0;
        /// <summary>貯水位</summary>
        public double m_dHdam = 0;
        /// <summary>貯水量</summary>
        public double m_dStorage = 0;
        /// <summary>流入量（１ステップ前）</summary>
        public double m_dQin = 0;


        /// <summary>ただし書き開始時の放流量</summary>
        public double m_dQ_tds = 0;
        /// <summary>フラグ</summary>
        public bool m_bflg_tds = false;

        // 出力情報制御追記20140925
        /// <summary>放流量出力数</summary>
        public long m_lOutDataQNum = 0;
        /// <summary>放流量出力のインデックス</summary>
        public long[] m_lOutQIndx = null;
        /// <summary>貯水位出力数</summary>
        public long m_lOutDataHNum = 0;
        /// <summary>貯水位出力のインデックス</summary>
        public long[] m_lOutHIndx = null;
        /// <summary>貯留量入力数</summary>
        public long m_lOutDataVNum = 0;
        /// <summary>貯留量出力のインデックス</summary>
        public long[] m_lOutVIndx = null;



        /// <summary><para>method outline:</para>
        /// <para>自己複製</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para> Clone( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>同一内容で　別インスタンス</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
        /// </remarks>
        public override McCalInfo Clone()
        {
            DamConstantRateDischargingCalInfo csRtn = new DamConstantRateDischargingCalInfo();
            csRtn.CopyInfo(this);
            return csRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>引数で与えられた情報を自分にコピーを行う</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
        /// </example>
        /// <param name="csOrgInfo">コピー元情報</param>
        /// <returns> bool true :正常 , false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override bool CopyInfo(McCalInfo csOrgInfo)
        {
            DamConstantRateDischargingCalInfo csOrdDt = (DamConstantRateDischargingCalInfo)csOrgInfo;

            this.m_dQc = csOrdDt.m_dQc;
            this.m_da = csOrdDt.m_da;
            this.m_dQmax = csOrdDt.m_dQmax;
            this.m_dDamH1 = csOrdDt.m_dDamH1;
            this.m_bato = csOrdDt.m_bato;
            this.m_dDamH2 = csOrdDt.m_dDamH2;

            this.m_dInit = csOrdDt.m_dInit;

            this.m_sHvFile = csOrdDt.m_sHvFile;

            this.m_ddamH = new double[this.m_ldamHV];
            this.m_ddamV = new double[this.m_ldamHV];
            for (long l = 0; l < m_ldamHV; l++)
            {
                this.m_ddamH[l] = csOrdDt.m_ddamH[l];
                this.m_ddamV[l] = csOrdDt.m_ddamV[l];
            }

            this.m_dQout = csOrdDt.m_dQout;
            this.m_dHdam = csOrdDt.m_dHdam;
            this.m_dStorage = csOrdDt.m_dStorage;

            this.m_dQin = csOrdDt.m_dQin;

            this.m_dQ_tds = csOrdDt.m_dQ_tds;
            this.m_bflg_tds = csOrdDt.m_bflg_tds;

            return true;
        }

    }
}
